#include "CppOsTest.h"

using namespace std;

namespace testing
{
	bool failed = false;
	std::vector<Test *> AllTests;
	int passCount = 0;
	std::vector<Test *> Fails;
	EOut ErrorOutput;
	TEOut ThrowErrorOutput;
	DOut DummyOutput;
	
	Test::Test()
	{
	}
	
	Test::~Test()
	{
		if (FullName)
			delete[] FullName;
	}

	void Test::SetUp()
	{
	}
	
	void Test::TearDown()
	{
	}

	const char *Test::GetFullName()
	{
		return FullName;
	}

	void Test::SetName()
	{
		if (!CaseName || !TestName)
			return;

		FullName = new char[strlen(CaseName) + 1 + strlen(TestName) + 1];
		strcpy(FullName, CaseName);
		strcat(FullName, ".");
		strcat(FullName, TestName);
	}
	
	void AddTest(Test *tc)
	{
		AllTests.push_back(tc);
	}

	static void RunTest(Test &t)
	{
		printf("[ RUN      ] %s\n", t.GetFullName());

		try
		{
			t.SetUp();
			t.RunTest();
		}
		catch (...)
		{
		}
		t.TearDown();

		if (failed)
		{
			printf("[  FAILED  ] %s\n", t.GetFullName());
			Fails.push_back(&t);
		}
		else
		{
			printf("[       OK ] %s\n", t.GetFullName());
			passCount++;
		}
	}
	
	static void PrintResults()
	{
		printf("[  PASSED  ] %d tests\n", passCount);
		if (::testing::Fails.size() != 0)
		{
			printf("[  FAILED  ] %d tests, listed below:\n");
			for (size_t i = 0; i < ::testing::Fails.size(); i++)
			{
				printf("[  FAILED  ] %s\n", Fails[0]->GetFullName());
			}
		}	
	}
	
	int doRun()
	{
		for (size_t i = 0; i < AllTests.size(); i++)
		{
			failed = false;
			Test &t = *AllTests[i];
			
			RunTest(t);
		}
		
		PrintResults();
		
		return Fails.size() != 0;
	}
	
	void hasFailed()
	{
		failed = true;
	}

	Out *eqFail(const char *expected, const char *actual, int line, const char *file, bool fail)
	{
		hasFailed();

		printf("%s:%d: Failure\n", file, line);
		printf("Value of: %d\n", actual);
		printf("Expected: %d\n", expected);

		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}

	Out *genericFail(const char *x, const char *y, const char *symbol, int line, char *file, bool fail)
	{
		hasFailed();

		printf("%s:%d: Failure\n", file, line);
		printf("Expected: (%s) %s (%s), actual: %s vs %s\n)", x, symbol, y, x, y);

		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *booleanFail(bool actual, bool expected, const char *condition, int line, char *file, bool fail)
	{
		hasFailed();
		
		printf("%s:%d: Failure\n", file, line);
		printf("Value of: %s\n", condition);
		printf("  Actual: %s\n", (actual ? "true" : "false"));
		printf("Expected: %s\n", (expected ? "true" : "false"));
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	char *strtolower(const char *str1)
	{
		int len = strlen(str1);
		char *str2 = new char[len];
		for (int i = 0; i < len; i++)
		{
			str2[i] = tolower(str1[i]);
		}
		str2[len] = 0;
		
		return str2;
	}
	
	bool doStringTest(const char *str1, const char *str2, bool equality, bool ignoreCase)
	{
		if (!str1)
		{
			if (!str2)
			{
				if (equality)
					return true;
				return false;
			}
			
			if (equality)
				return false;
			return true;
		}
		
		if (ignoreCase)
		{
			str1 = strtolower(str1);
			str2 = strtolower(str2);
		}
		
		int result = strcmp(str1, str2);
		
		if (ignoreCase)
		{
			delete[] str1;
			delete[] str2;
		}
		
		if (equality)
			return result == 0;
		return result != 0;
	}
	
	Out *stringFail(const char *expected, const char *actual, bool equality, bool ignoreCase, int line, char *file, bool fail)
	{
		hasFailed();
		
		printf("%s:%d: Failure\n", file, line);
		if (equality)
		{
			printf("Value of: %s\n", actual);
			printf("Expected: %s%s\n", expected, (ignoreCase ? " (ignoring case)" : ""));
		}
		else
		{
			printf("Expected: (%s) != (%s), actual: %s vs %s\n", expected, actual, expected, actual);
		}
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	
	}
	
	Out *excFail(const char *expr, const char *exc, int line, char *file, bool fail)
	{
		hasFailed();
		printf("%s:%d: Failure\n", file, line);
		printf("Expected: %s throws an exception of type %s.\n", expr, exc);
		printf("  Actual: it throws nothing.\n");
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}

	Out *excFail2(const char *expr, int line, char *file, bool fail)
	{
		hasFailed();
		printf("%s:%d: Failure\n", file, line);
		printf("Expected: %s throws an exception.\n", expr);
		printf("  Actual: it doesn't.\n");
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *excFail3(const char *expr, int line, char *file, bool fail)
	{
		hasFailed();
		printf("%s:%d: Failure\n", file, line);
		printf("Expected: %s doesn't throw an exception.", expr);
		printf("  Actual: it throws.\n");
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred1Fail(const char *fn, int val1, const char *val1str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		printf("%s:%d: Failure\n", file, line);
		printf("%s(%s) evaluates to false, where:\n", fn, val1str);
		printf("%s evaluates to %d", val1str, val1);
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred2Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		printf("%s:%d: Failure\n", file, line);
		printf("%s(%s, %s) evaluates to false, where:\n", fn, val1str, val2str);
		printf("%s evaluates to %d", val1str, val1);
		printf("%s evaluates to %d", val2str, val2);
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred3Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		printf("%s:%d: Failure\n", file, line);
		printf("%s(%s, %s, %s) evaluates to false, where:\n", fn, val1str, val2str, val3str);
		printf("%s evaluates to %d", val1str, val1);
		printf("%s evaluates to %d", val2str, val2);
		printf("%s evaluates to %d", val3str, val3);
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred4Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int val4, const char *val4str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		printf("%s:%d: Failure\n", file, line);
		printf("%s(%s, %s, %s, %s) evaluates to false, where:\n", fn, val1str, val2str, val3str, val4str);
		printf("%s evaluates to %d", val1str, val1);
		printf("%s evaluates to %d", val2str, val2);
		printf("%s evaluates to %d", val3str, val3);
		printf("%s evaluates to %d", val4str, val4);
		
		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
	
	Out *pred5Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int val4, const char *val4str, int val5, const char *val5str, int line, const char *file, bool fail)
	{
		hasFailed();
		
		printf("%s:%d: Failure\n", file, line);
		printf("%s(%s, %s, %s, %s, %s) evaluates to false, where:\n", fn, val1str, val2str, val3str, val4str, val5str);
		printf("%s evaluates to %d", val1str, val1);
		printf("%s evaluates to %d", val2str, val2);
		printf("%s evaluates to %d", val3str, val3);
		printf("%s evaluates to %d", val4str, val4);
		printf("%s evaluates to %d", val5str, val5);

		if (fail)
			return (Out *) &ThrowErrorOutput;
		return (Out *) &ErrorOutput;
	}
}
